/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.myfaces.orchestra.lib.jsf;

import org.apache.myfaces.orchestra.frameworkAdapter.FrameworkAdapter;

import javax.faces.FacesException;
import javax.faces.application.Application;
import javax.faces.application.NavigationHandler;
import javax.faces.application.StateManager;
import javax.faces.application.ViewHandler;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.el.MethodBinding;
import javax.faces.el.PropertyResolver;
import javax.faces.el.ReferenceSyntaxException;
import javax.faces.el.ValueBinding;
import javax.faces.el.VariableResolver;
import javax.faces.event.ActionListener;
import javax.faces.validator.Validator;
import javax.el.ELResolver;
import javax.el.ValueExpression;
import javax.el.ExpressionFactory;
import javax.el.ELContextListener;
import javax.el.ELException;
import java.util.Collection;
import java.util.Iterator;
import java.util.Locale;
import java.util.ResourceBundle;

/**
 * Allows a better spring configuration, currently it allows one to:
 * <ul>
 *	<li>configure a converter in the spring configuration to get in use of the persistence dependency injections</li>
 * </ul>
 */

// There is a technical problem here: Orchestra should be usable on both JSF1.1 and JSF1.2.
// Ideally we would compile against JSF1.1, and JSF1.2 would be backwards-compatible with it.
// Unfortunately, that is surprisingly hard to achieve. Any suggestions welcome!
// In the meantime, we compile against JSF1.2 then test against JSF1.1 to verify compatibility :-(.

public class OrchestraApplication extends Application
{
	private final Application original;

	public OrchestraApplication(Application original)
	{
		this.original = original;
	}

	public Converter createConverter(String converterId)
	{
		try
		{
			return original.createConverter(converterId);
		}
		catch (FacesException e)
		{
			// looks like there is no converter configured, try to find a spring-configured one
			Converter converter = (Converter) FrameworkAdapter.getCurrentInstance().getBean(converterId);
			if (converter == null)
			{
				// no spring converter ... now re-throw the exception
				throw e;
			}

			return new SerializableConverter(converterId, converter);
		}
	}

	public Converter createConverter(Class aClass)
	{
		return original.createConverter(aClass);
	}

	public Iterator getConverterIds()
	{
		return original.getConverterIds();
	}

	public Iterator getConverterTypes()
	{
		return original.getConverterTypes();
	}

	public MethodBinding createMethodBinding(String s, Class[] classes)
		throws ReferenceSyntaxException
	{
		return original.createMethodBinding(s, classes);
	}

	public Iterator getSupportedLocales()
	{
		return original.getSupportedLocales();
	}

	public void setSupportedLocales(Collection locales)
	{
		original.setSupportedLocales(locales);
	}

	public void addValidator(String s, String s1)
	{
		original.addValidator(s, s1);
	}

	public Validator createValidator(String s)
		throws FacesException
	{
		return original.createValidator(s);
	}

	public Iterator getValidatorIds()
	{
		return original.getValidatorIds();
	}

	public ValueBinding createValueBinding(String s)
		throws ReferenceSyntaxException
	{
		return original.createValueBinding(s);
	}

	public void addELResolver(ELResolver elResolver)
	{
		original.addELResolver(elResolver);
	}

	public ELResolver getELResolver()
	{
		return original.getELResolver();
	}

	public ResourceBundle getResourceBundle(FacesContext facesContext, String s)
		throws FacesException, NullPointerException
	{
		return original.getResourceBundle(facesContext, s);
	}

	public UIComponent createComponent(ValueExpression valueExpression, FacesContext facesContext, String s)
		throws FacesException, NullPointerException
	{
		return original.createComponent(valueExpression, facesContext, s);
	}

	public ExpressionFactory getExpressionFactory()
	{
		return original.getExpressionFactory();
	}

	public void addELContextListener(ELContextListener elContextListener)
	{
		original.addELContextListener(elContextListener);
	}

	public void removeELContextListener(ELContextListener elContextListener)
	{
		original.removeELContextListener(elContextListener);
	}

	public ELContextListener[] getELContextListeners()
	{
		return original.getELContextListeners();
	}

	public Object evaluateExpressionGet(FacesContext facesContext, String s, Class aClass)
		throws ELException
	{
		return original.evaluateExpressionGet(facesContext, s, aClass);
	}

	public ActionListener getActionListener()
	{
		return original.getActionListener();
	}

	public void setActionListener(ActionListener actionListener)
	{
		original.setActionListener(actionListener);
	}

	public Locale getDefaultLocale()
	{
		return original.getDefaultLocale();
	}

	public void setDefaultLocale(Locale locale)
	{
		original.setDefaultLocale(locale);
	}

	public String getDefaultRenderKitId()
	{
		return original.getDefaultRenderKitId();
	}

	public void setDefaultRenderKitId(String s)
	{
		original.setDefaultRenderKitId(s);
	}

	public String getMessageBundle()
	{
		return original.getMessageBundle();
	}

	public void setMessageBundle(String s)
	{
		original.setMessageBundle(s);
	}

	public NavigationHandler getNavigationHandler()
	{
		return original.getNavigationHandler();
	}

	public void setNavigationHandler(NavigationHandler navigationHandler)
	{
		original.setNavigationHandler(navigationHandler);
	}

	public PropertyResolver getPropertyResolver()
	{
		return original.getPropertyResolver();
	}

	public void setPropertyResolver(PropertyResolver propertyResolver)
	{
		original.setPropertyResolver(propertyResolver);
	}

	public VariableResolver getVariableResolver()
	{
		return original.getVariableResolver();
	}

	public void setVariableResolver(VariableResolver variableResolver)
	{
		original.setVariableResolver(variableResolver);
	}

	public ViewHandler getViewHandler()
	{
		return original.getViewHandler();
	}

	public void setViewHandler(ViewHandler viewHandler)
	{
		original.setViewHandler(viewHandler);
	}

	public StateManager getStateManager()
	{
		return original.getStateManager();
	}

	public void setStateManager(StateManager stateManager)
	{
		original.setStateManager(stateManager);
	}

	public void addComponent(String s, String s1)
	{
		original.addComponent(s, s1);
	}

	public UIComponent createComponent(String s)
		throws FacesException
	{
		return original.createComponent(s);
	}

	public UIComponent createComponent(ValueBinding valueBinding, FacesContext facesContext, String s)
		throws FacesException
	{
		return original.createComponent(valueBinding, facesContext, s);
	}

	public Iterator getComponentTypes()
	{
		return original.getComponentTypes();
	}

	public void addConverter(String s, String s1)
	{
		original.addConverter(s, s1);
	}

	public void addConverter(Class aClass, String s)
	{
		original.addConverter(aClass, s);
	}
}
