/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.myfaces.orchestra.lib.jsf;

import org.aopalliance.intercept.MethodInvocation;
import org.apache.myfaces.orchestra.frameworkAdapter.FrameworkAdapter;

import javax.faces.FacesException;
import javax.faces.application.Application;
import javax.faces.application.ApplicationFactory;
import javax.faces.convert.Converter;

/**
 * Decorate the orignial application and provide an Orchestra enhanced version.
 *
 * @see OrchestraApplication
 */
public class OrchestraApplicationFactory extends ApplicationFactory
{
	private final ApplicationFactory original;

	private Application application;

	public OrchestraApplicationFactory(ApplicationFactory original)
	{
		this.original = original;
	}

	public Application getApplication()
	{
		if (application == null)
		{
			application = new OrchestraApplication(original.getApplication());
		}

		return application;
	}

	public void setApplication(Application application)
	{
		original.setApplication(application);
	}

	public Converter createConverter(String converterId, MethodInvocation methodInvocation) throws Throwable
	{
		try
		{
			return (Converter) methodInvocation.proceed();
		}
		catch (FacesException e)
		{
			// looks like there is no converter configured, try to find a spring-configured one
			Converter converter = (Converter) FrameworkAdapter.getCurrentInstance().getBean(converterId);
			if (converter == null)
			{
				// no spring converter ... now re-throw the exception
				throw e;
			}

			return new SerializableConverter(converterId, converter);
		}
	}
}